/*
 *  arch/arm/mach-rpc/include/mach/timex.h
 *
 *  Copyright (C) 1997, 1998 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  RiscPC architecture timex specifications
 */

/*
 * On the RiscPC, the clock ticks at 2MHz.
 */
#define CLOCK_TICK_RATE		2000000

