/*
 * Copyright (C) 1999, 2000  Niibe Yutaka  &  Kaz Kojima
 * Copyright (C) 2002 Paul Mundt
 */
#ifndef __ASM_SH_EXEC_H
#define __ASM_SH_EXEC_H

#define arch_align_stack(x) (x)

#endif /* __ASM_SH_EXEC_H */
