#ifndef __ASM_SH_CPU_SH2A_RTC_H
#define __ASM_SH_CPU_SH2A_RTC_H

#define rtc_reg_size		sizeof(u16)
#define RTC_BIT_INVERTED	0
#define RTC_DEF_CAPABILITIES	RTC_CAP_4_DIGIT_YEAR

#endif /* __ASM_SH_CPU_SH2A_RTC_H */
